#include "particle.h"
#include "util.h"
#include <windows.h>
#include <GL/gl.h>
#include <math.h>

ParticleEmitterSet emitterSet;

ParticleEmitter::ParticleEmitter(unsigned int texId, double w, double h, double tpp, double ttl, float col1[3], float col2[3])
{
	tex = texId;
	width = w;
	height = h;

	curTime = 0;
	timeSinceLast = 0;
	timePerParticle = tpp;
	timeToLive = ttl;
	killed = false;

	for (int i = 0; i < 3; i++)
	{
		color1[i] = col1[i];
		color2[i] = col2[i];
	}
}

ParticleEmitter* ParticleEmitter::Clone()
{
	return new ParticleEmitter(*this);
}

void ParticleEmitter::Kill()
{
	killed = true;
}

bool ParticleEmitter::UpdateParticle(Particle& p, double timeElapsed)
{
	p.pos[0] += p.vel[0] * timeElapsed;
	p.pos[1] += p.vel[1] * timeElapsed;
	p.pos[2] += p.vel[2] * timeElapsed;
	p.life -= p.fade * timeElapsed;

	return p.life > 0;
}

bool ParticleEmitter::Update(double timeElapsed)
{
	// Add time from last
	timeSinceLast += timeElapsed;
	curTime += timeElapsed;

	if (!killed)
	{
		// Construct particle if time elapsed
		while (timeSinceLast > timePerParticle)
		{
			particles.push_back(CreateParticle(x,y,z));
			timeSinceLast -= timePerParticle;
		}
	} else {
		if (particles.empty())
			return false;
	}

	lastPos[0] = x;
	lastPos[1] = y;
	lastPos[2] = z;

	for (ParticleIt it = particles.begin(); it != particles.end();)
	{
		bool live = UpdateParticle(*it, timeElapsed);

		if (!live)
			particles.erase(it++);
		else
			++it;
	}

	return true;
}



Particle ParticleEmitter::CreateParticle(float x, float y, float z)
{
	float col[3];
	float t = randf();
	for (int i = 0; i < 3; i++)
		col[i] = (1-t)*color1[i] + t*color2[i];

	Particle p = {
		{x, y, z}, 
		{col[0], col[1], col[2]},
		{randf()*100.0 - 50.0, -randf()*100.0, 0},
		1.0, 1.0 / timeToLive};

	return p;
}


void ParticleEmitter::Draw()
{
	// Setup blending
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

	// Setup texture
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, tex);

	// Draw particles
	float w_2 = width/2.0;
	float h_2 = height/2.0;

	glBegin(GL_QUADS);
		for (ParticleIt it = particles.begin(); it != particles.end(); ++it)
		{
			glColor4f(it->col[0], it->col[1], it->col[2], it->life);
			// glColor3fv(it->col);
			typedef float Point[3];
			Point& pos = it->pos;

			glTexCoord2f(0, 1);
			glVertex3f(pos[0] - w_2, pos[1] - h_2, pos[2]);

			glTexCoord2f(1, 1);
			glVertex3f(pos[0] + w_2, pos[1] - h_2, pos[2]);

			glTexCoord2f(1, 0);
			glVertex3f(pos[0] + w_2, pos[1] + h_2, pos[2]);

			glTexCoord2f(0, 0);
			glVertex3f(pos[0] - w_2, pos[1] + h_2, pos[2]);
		}
	glEnd();

}




SpinEmitter::SpinEmitter(unsigned int texId, double w, double h, double tpp, double ttl, float col1[3], float col2[3], float r, float freq)
  : ParticleEmitter(texId, w, h, tpp, ttl, col1, col2)
{
	radius = r;
	frequency = freq;
}


ParticleEmitter* SpinEmitter::Clone()
{
	return new SpinEmitter(*this);
}

Particle SpinEmitter::CreateParticle(float x, float y, float z)
{
	Particle p = ParticleEmitter::CreateParticle(x,y,z);

	static const double PI = 2*acos(0.0);
	
	double dif = sin((curTime * 2*PI) * frequency) * radius;

	float dir[3] = {x - lastPos[0], y - lastPos[1], z - lastPos[2]};
	if (dir[0] == 0 && dir[1] == 0 && dir[2] == 0) {
		dir[1] = 1.0;
	} else {
		vecNormalize(dir);
	}

	p.pos[0] += dif*dir[1];
	p.pos[1] += -dif*dir[0];

	/*p.vel[0] = 0;
	p.vel[1] = 0;*/

	p.vel[0] = -dif*dir[1];
	p.vel[1] = dif*dir[0];

	return p;
}





void ParticleEmitterSet::Update(double timeDif)
{
	for (EmitterIt it = emitters.begin(); it != emitters.end();)
	{
		if (!(*it)->Update(timeDif))
		{
			delete *it;
			emitters.erase(it++);
		} else {
			++it;
		}
	}
}

void ParticleEmitterSet::Draw()
{
	for (EmitterIt it = emitters.begin(); it != emitters.end();++it)
	{
		(*it)->Draw();
	}
}

void ParticleEmitterSet::Add(ParticleEmitter* e)
{
	emitters.push_back(e);
}


